"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tapStartToRunWorkers = void 0;
const files_change_1 = require("../files-change");
const infrastructure_logger_1 = require("../infrastructure-logger");
const interceptDoneToGetWebpackDevServerTap_1 = require("./interceptDoneToGetWebpackDevServerTap");
const pluginHooks_1 = require("./pluginHooks");
const pluginPools_1 = require("./pluginPools");
const tapAfterCompileToGetIssues_1 = require("./tapAfterCompileToGetIssues");
const tapDoneToAsyncGetIssues_1 = require("./tapDoneToAsyncGetIssues");
function tapStartToRunWorkers(compiler, getIssuesWorker, getDependenciesWorker, config, state) {
    const hooks = pluginHooks_1.getForkTsCheckerWebpackPluginHooks(compiler);
    const { log, debug } = infrastructure_logger_1.getInfrastructureLogger(compiler);
    compiler.hooks.run.tap('ForkTsCheckerWebpackPlugin', () => {
        if (!state.initialized) {
            debug('Initializing plugin for single run (not async).');
            state.initialized = true;
            state.watching = false;
            tapAfterCompileToGetIssues_1.tapAfterCompileToGetIssues(compiler, config, state);
        }
    });
    compiler.hooks.watchRun.tap('ForkTsCheckerWebpackPlugin', () => __awaiter(this, void 0, void 0, function* () {
        if (!state.initialized) {
            state.initialized = true;
            state.watching = true;
            if (config.async) {
                debug('Initializing plugin for watch run (async).');
                tapDoneToAsyncGetIssues_1.tapDoneToAsyncGetIssues(compiler, config, state);
                interceptDoneToGetWebpackDevServerTap_1.interceptDoneToGetWebpackDevServerTap(compiler, config, state);
            }
            else {
                debug('Initializing plugin for watch run (not async).');
                tapAfterCompileToGetIssues_1.tapAfterCompileToGetIssues(compiler, config, state);
            }
        }
    }));
    compiler.hooks.compilation.tap('ForkTsCheckerWebpackPlugin', (compilation) => __awaiter(this, void 0, void 0, function* () {
        if (compilation.compiler !== compiler) {
            // run only for the compiler that the plugin was registered for
            return;
        }
        const iteration = ++state.iteration;
        let change = {};
        if (state.watching) {
            change = files_change_1.getFilesChange(compiler);
            log([
                'Calling reporter service for incremental check.',
                `  Changed files: ${JSON.stringify(change.changedFiles)}`,
                `  Deleted files: ${JSON.stringify(change.deletedFiles)}`,
            ].join('\n'));
        }
        else {
            log('Calling reporter service for single check.');
        }
        change = yield hooks.start.promise(change, compilation);
        debug(`Submitting the getIssuesWorker to the pool, iteration ${iteration}.`);
        state.issuesPromise = pluginPools_1.issuesPool.submit(() => __awaiter(this, void 0, void 0, function* () {
            try {
                debug(`Running the getIssuesWorker, iteration ${iteration}.`);
                return yield getIssuesWorker(change, state.watching);
            }
            catch (error) {
                hooks.error.call(error, compilation);
                return undefined;
            }
            finally {
                debug(`The getIssuesWorker finished its job, iteration ${iteration}.`);
            }
        }));
        debug(`Submitting the getDependenciesWorker to the pool, iteration ${iteration}.`);
        state.dependenciesPromise = pluginPools_1.dependenciesPool.submit(() => __awaiter(this, void 0, void 0, function* () {
            try {
                debug(`Running the getDependenciesWorker, iteration ${iteration}.`);
                return yield getDependenciesWorker(change);
            }
            catch (error) {
                hooks.error.call(error, compilation);
                return undefined;
            }
            finally {
                debug(`The getDependenciesWorker finished its job, iteration ${iteration}.`);
            }
        }));
    }));
}
exports.tapStartToRunWorkers = tapStartToRunWorkers;
